/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/
/*


08/01/02	-	endre

		TOMI Paradigm Independent API ported from Tihamer's
		graph rewriting engine. See documentation for API description.

		Headers of API calls are added as member functions to 
		class Udm::Object

07/31/02	-	endre


		ChildrenAttr:
				
					- added cast operator for vector<CLASS> to preserve 
						the order of the children. This leaves all previous
						set<> assignments intact, except this note:

					
					- +=, -= operators on children now return vectors instead
						of sets. This should not be a problem when using a syntax 
						like this:

							lamp.children() += new_bulb;

						because there is a ChildrenAttr assignment operator 
						which takes a vector of object as input. 

						However, if any of these syntax is used, 
						the code will not compile:

						1:
							[...]
							set<Bulb> bulbs = lamp.children() += new_bulb;
							[...]
						
						2:
							[...]
							set<Bulb> AddBulbFunc(Bulb newBulb, Lamp hostLamp)
							{
								return hostLamp.children() += new_bulb;
							}
							[...]

		

						the sets should be changed to vectors, when using 
						the return value of the operators

						The change would be:
						
						1:
							[...]
							lamp.children() += new_bulb;
							set<Bulb> bulbs = lamp.children();
							[...]
						
						2:

							[...]
							set<Bulb> AddBulbFunc(Bulb newBulb, Lamp hostLamp)
							{
								lamp.children() += new_bulb;
								return lamp.children();
							}
							[...]


													

					

06/19/02	-	endre
	
	  Attribute array support: 
	  
		Added Attribute Array classes for both persistent and non-persistent
		attributes.

		Added array attribute setter/getter declarations for ObjectImpl, for both
		persistent and non-persistent attributes

		However, the persistent array attribute setters/getters
		are not abstract functions, a basic functionality is 
		provided at ObjectImpl level, but it can be overriden with
		a more performant one in the backend specific ObjectImpl class.
		See comments in UdmBase.cpp

*/
#ifndef MOBIES_UDMBASE_H
#define MOBIES_UDMBASE_H

#pragma warning ( disable : 4786 )
#pragma warning( disable : 4290 ) //C++ Exception Specification Ignored warning
#pragma warning( disable : 4503 ) //Decorated name length exceeded
#pragma warning( disable : 4103 ) //used #pragma pack to change alignment

#ifndef MOBIES_ERRHAND_H
#include "ErrHand.h"
#endif


#include <set>
#include <vector>
#include <list>
#include <string>
#include <map>

namespace Uml
{
	class Diagram;
	class Class;
	class Attribute;
	class Association;
	class AssociationRole;
	class Composition;
	class CompositionParentRole;
	class CompositionChildRole;
	const AssociationRole &theOther1(const AssociationRole &role);
};

namespace Udm
{
// --------------------------- ObjectImpl

	// NOTE: All returned ObjectImpl pointers are refCounted (if not NULL),
	// they must be released, or passed to the Object constructor.

	static const int TARGETFROMPEER = 0;
	static const int TARGETFROMCLASS = 1;
	static const int CLASSFROMTARGET = 2;

	class DataNetwork;
	class udmvariant;

	typedef map<string, udmvariant> tmap;
	bool IsDerivedFrom(const Uml::Class &derived, const Uml::Class &base);
	
	class ObjectImpl
	{
	// --- reference counting

	public:
		virtual ObjectImpl *clone() = 0;
		virtual void release() = 0;
		virtual DataNetwork *__getdn() = 0;

	// --- type

	public:
		virtual const Uml::Class &type() const = 0;

	// --- order

	public:
		// every object in one project must return a unique uniqueId
		typedef long uniqueId_type;

		virtual uniqueId_type uniqueId() const = 0;

	// --- attributes

	public:
		//no array versions
		virtual string getStringAttr(const Uml::Attribute &meta) const = 0;
		virtual void setStringAttr(const Uml::Attribute &meta, const string &a, const bool direct = true) = 0;

		virtual bool getBooleanAttr(const Uml::Attribute &meta) const = 0;
		virtual void setBooleanAttr(const Uml::Attribute &meta, bool a, const bool direct = true) = 0;

		virtual __int64 getIntegerAttr(const Uml::Attribute &meta) const = 0;
		virtual void setIntegerAttr(const Uml::Attribute &meta, __int64 a, const bool direct = true) = 0;

		virtual double getRealAttr(const Uml::Attribute &meta) const = 0;
		virtual void setRealAttr(const Uml::Attribute &meta, double a, const bool direct = true) = 0;

		//array versions

		virtual vector<string> getStringAttrArr(const Uml::Attribute &meta) const;
		virtual void setStringAttrArr(const Uml::Attribute &meta, const vector<string> &a, const bool direct = true);

		virtual vector<bool> getBooleanAttrArr(const Uml::Attribute &meta) const;
		virtual void setBooleanAttrArr(const Uml::Attribute &meta, const vector<bool> &a, const bool direct = true);

		virtual vector<__int64> getIntegerAttrArr(const Uml::Attribute &meta) const;
		virtual void setIntegerAttrArr(const Uml::Attribute &meta, const vector<__int64> &a, const bool direct = true) ;

		virtual vector<double> getRealAttrArr(const Uml::Attribute &meta) const;
		virtual void setRealAttrArr(const Uml::Attribute &meta, const vector<double> &a, const bool direct = true);




	// --- containment

	public:

// get the parent object or null if object has no parent
// if role is not NULLROLE, return parent only if the role actually applies
		virtual ObjectImpl *getParent(const Uml::CompositionParentRole &role) const = 0;	// can return NULL

// role must be specified unless a is null, or child role in the new parent is unambigious
// if a was already the parent, existing other childroles are not deleted
// XML: if multiple roles are enabled, 
//		object will keep its existing role(s) as well if not moved to a new parent
//		if a = NULL and role != null, the role is removed from the containment relation (if it was there), 
//									   but the object will only be deleted, if the role specified was its only one
		virtual void setParent(ObjectImpl *a, const Uml::CompositionParentRole &role, const bool direct = true) = 0;

// set parent to null; equivalent to setParent(NULL, NULL);
		virtual void detach() = 0;

// if role is given, return only those children that have that role (ignore kind)
// else if kind is not null, return all children which are compatible with kind
// else if kind is null, return all children
		virtual vector<ObjectImpl*> getChildren(const Uml::CompositionChildRole &role, const Uml::Class &kind) const = 0;

// if role is non-null. 
//		All children omitted from the list lose this role and may also
//		become detached if this was their only role.
// if role is null
//		All existing children are removed and new children are added only if their containment role is unambigious
//	XML: if multiple roles are enabled, new children with this role may already have other roles as well.

// direct specifies if the call is coming from user code (true), or is a recursive call
// which was invoked because this operation was invoked on an archetype
		virtual void setChildren(const Uml::CompositionChildRole &meta, const vector<ObjectImpl*> &a, const bool direct = true) = 0;


// role must be specified unless child role in the new parent is unambigious

		/*
				if there is an archetype, the 3rd argument is not null
				if arg exists,
					if subtype is false, instance is created (default behaviour)
					if subtype is true, subtype/derived object is created

				if real_archetype is false, the child being created is created automatically,
				upon createChild() of the parent, because the parent is an instace/derived
				of an archetype which has children
		*/
		virtual ObjectImpl *createChild(const Uml::CompositionChildRole &role, 
			const Uml::Class &meta,
			const ObjectImpl* arch = NULL, 
			const bool subtype = false,
			const bool real_archetype = true) = 0;


		/* 
			should be invoked when an object has an archetype
			returns false if the object is there because one of the parent inherited it
			returns true if the object was instantiated/derived directly, from an archetype
		*/
		virtual bool hasRealArchetype() const = 0;
	// --- associations

	public:
// arole must be non-null. 
// Associations with names on both sides are navigable from both directions.
		virtual vector<ObjectImpl*> getAssociation(const Uml::AssociationRole &arole, int mode = TARGETFROMPEER) const = 0;
// arole must be non-null. 
// Existing associations omitted from the new list are 
// deleted, along with their association classes.
// Associations created from either side are considered equal


		virtual void setAssociation(
			const Uml::AssociationRole &arole, 
			const vector<ObjectImpl*> &a, 
			int mode = TARGETFROMPEER,
			const bool direct = true) = 0;	

		//if direct is false, the link to be created is an inherited link
		//that exists between the children of my parent's archetype
		//and needs to be copied here 

	//archetype/derived relationships
		virtual vector<ObjectImpl*> getDerived() const = 0;
		virtual vector<ObjectImpl*> getInstances() const = 0;
		virtual ObjectImpl* getArchetype() const = 0;
	
// temporary values
		tmap &locatemap(bool create) const;

		//single value attributes

		string getTempStringAttr(const Uml::Attribute &meta) const;
		void setTempStringAttr(const Uml::Attribute &meta, const string &a, const bool direct = true);
		__int64 getTempIntegerAttr(const Uml::Attribute &meta) const;
		void setTempIntegerAttr(const Uml::Attribute &meta, const __int64 &a, const bool direct = true);
		double getTempRealAttr(const Uml::Attribute &meta) const;
		void setTempRealAttr(const Uml::Attribute &meta, const double &a, const bool direct = true);
		bool getTempBooleanAttr(const Uml::Attribute &meta) const;
		void setTempBooleanAttr(const Uml::Attribute &meta, const bool &a, const bool direct = true);

		//multiple value attributes

		vector<string> getTempStringAttrArr(const Uml::Attribute &meta) const;
		void setTempStringAttrArr(const Uml::Attribute &meta, const vector<string> &a, const bool direct = true);
		vector<__int64> getTempIntegerAttrArr(const Uml::Attribute &meta) const;
		void setTempIntegerAttrArr(const Uml::Attribute &meta, const vector<__int64> &a, const bool direct = true);
		vector<double> getTempRealAttrArr(const Uml::Attribute &meta) const;
		void setTempRealAttrArr(const Uml::Attribute &meta, const vector<double> &a, const bool direct = true);
		vector<bool> getTempBooleanAttrArr(const Uml::Attribute &meta) const;
		void setTempBooleanAttrArr(const Uml::Attribute &meta, const vector<bool> &a, const bool direct = true);

		//for new Objects, set the default values for attributes
		//this will be invoked whenever creating a new ObjectImpl instance
		//in each backend
		//if all is false, only non-persistent attributes are set to default values
		//MEM backend has an optimized version for default values.
		void setDefaultAttributes(const bool all = true);

		//atribute copier functions
		void CopyAttributeFrom(const Uml::Attribute& which, const ObjectImpl* from);
		void CopyAttributesFrom(const ObjectImpl*  from);
		void CopyAttributeFromArchetype(const Uml::Attribute& which);
		void CopyAttributesFromArchetype();
		
		
	};


	typedef map<ObjectImpl::uniqueId_type, tmap> ttmap;
	
	//non-persistent, single-value attribute objects
	class TempStringAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempStringAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator string() const { return impl->getTempStringAttr(meta); }
		const string &operator =(const string &a) { impl->setTempStringAttr(meta, a); return a; }
	};



	class TempIntegerAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempIntegerAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator __int64() const { return impl->getTempIntegerAttr(meta); }
		const __int64 &operator =(const __int64 &a) { impl->setTempIntegerAttr(meta, a); return a; }
	};


	class TempRealAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempRealAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator double() const { return impl->getTempRealAttr(meta); }
		const double &operator =(const double &a) { impl->setTempRealAttr(meta, a); return a; }
	};


	class TempBooleanAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempBooleanAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator bool() const { return impl->getTempBooleanAttr(meta); }
		const bool &operator =(const bool &a) { impl->setTempBooleanAttr(meta, a); return a; }
	};
	




// --------------------------- null;

	class Object;
	extern Object null;

	class NullObject : public ObjectImpl
	{
	protected:
		static udm_exception e;

	public:
		inline NullObject();  
		// in Object::Object(...) we assume this refererence counting behaviour
		virtual ObjectImpl *clone() { return this; }
		virtual void release() { } 
		virtual DataNetwork *__getdn() { throw e; }

		virtual const Uml::Class &type() const { throw e; }

		virtual string getStringAttr(const Uml::Attribute &meta) const { throw e; }
		virtual void setStringAttr(const Uml::Attribute &meta, const string &a, const bool direct = true) { throw e; }

		virtual bool getBooleanAttr(const Uml::Attribute &meta) const { throw e; }
		virtual void setBooleanAttr(const Uml::Attribute &meta, bool a, const bool direct = true) { throw e; }

		virtual __int64 getIntegerAttr(const Uml::Attribute &meta) const { throw e; }
		virtual void setIntegerAttr(const Uml::Attribute &meta, __int64 a, const bool direct = true) { throw e; }

		virtual double getRealAttr(const Uml::Attribute &meta) const { throw e; }
		virtual void setRealAttr(const Uml::Attribute &meta, double a, const bool direct = true) { throw e; }

		virtual vector<string> getStringAttrArr(const Uml::Attribute &meta) const {throw e;};
		virtual void setStringAttrArr(const Uml::Attribute &meta, const vector<string> &a, const bool direct = true) {throw e;};

		virtual vector<bool> getBooleanAttrArr(const Uml::Attribute &meta) const {throw e;};
		virtual void setBooleanAttrArr(const Uml::Attribute &meta, const vector<bool> &a, const bool direct = true) {throw e;};

		virtual vector<__int64> getIntegerAttrArr(const Uml::Attribute &meta) const {throw e;};
		virtual void setIntegerAttrArr(const Uml::Attribute &meta, const vector<__int64> &a, const bool direct = true) {throw e;};

		virtual vector<double> getRealAttrArr(const Uml::Attribute &meta) const {throw e;};
		virtual void setRealAttrArr(const Uml::Attribute &meta, const vector<double> &a, const bool direct = true) {throw e;};




		virtual ObjectImpl *getParent(const Uml::CompositionParentRole &role) const { throw e; }
		virtual void setParent(ObjectImpl *a, const Uml::CompositionParentRole &role, const bool direct = true) { throw e; }
		virtual void detach() { throw e; }

		virtual vector<ObjectImpl*> getChildren(const Uml::CompositionChildRole &meta, const Uml::Class &kind) const { throw e; }
		virtual void setChildren(const Uml::CompositionChildRole &meta, const vector<ObjectImpl*> &a, const bool direct = true) { throw e; }

		virtual ObjectImpl *createChild(const Uml::CompositionChildRole &role,
			const Uml::Class &meta,
			const ObjectImpl* archetype = NULL,
			const bool subtype = false,
			const bool real_archetype = true) { throw e; }

		virtual vector<ObjectImpl*> getAssociation(const Uml::AssociationRole &meta, int mode = TARGETFROMPEER) const { throw e; }
		virtual void setAssociation(
			const Uml::AssociationRole &meta, 
			const vector<ObjectImpl*> &a, 
			int mode = TARGETFROMPEER, 
			const bool direct = true) { throw e; }

		virtual uniqueId_type uniqueId() const { return 0; }
		
		virtual vector<ObjectImpl*> getDerived() const {throw e;};
		virtual vector<ObjectImpl*> getInstances() const {throw e;};
		virtual ObjectImpl* getArchetype() const {throw e;};
		virtual bool hasRealArchetype() const {throw e;};
	

	};

	extern NullObject _null;

	inline NullObject::NullObject() { ASSERT(this == &_null); } 

	extern const Uml::CompositionChildRole NULLCHILDROLE;
	extern const Uml::CompositionParentRole NULLPARENTROLE;

// single-value(no-array) versions of Attribute objects

// --------------------------- StringAttr

	class StringAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		StringAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator string() const { return impl->getStringAttr(meta); }
		const string &operator =(const string &a) { impl->setStringAttr(meta, a); return a; }
		typedef const char * charp;
//		operator charp() const { return string().c_str(); } 
	};

// --------------------------- BooleanAttr

	class BooleanAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		BooleanAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator bool() const { return impl->getBooleanAttr(meta); }
		bool operator =(bool a) { impl->setBooleanAttr(meta, a); return a; }
	};

// --------------------------- IntegerAttr

	class IntegerAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		IntegerAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		//operator long() const { return impl->getIntegerAttr(meta); }
		operator __int64() const { return impl->getIntegerAttr(meta); }
		//long operator =(long a) { impl->setIntegerAttr(meta, a); return a; }
		__int64 operator =(__int64 a) { impl->setIntegerAttr(meta, a); return a; }
	};

// --------------------------- RealAttr

	class RealAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		RealAttr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator double() const { return impl->getRealAttr(meta); }
		double operator =(double a) { impl->setRealAttr(meta, a); return a; }
	};

// multiple-value(array) versions of Attribute objects
/*
	Array element template class to be returned by [] op,
	implementing the basic cast and assignment operators
*/	

	bool CheckAttributeOrdering(const Uml::Attribute & att);

	template <class CLASS>
	class ArrAttrItem
	{
		
	protected:
		
		const Uml::Attribute & meta;			//attribute to be altered
		vector<CLASS> array;		//vector with array elements
		ObjectImpl * object;	//object this array attribute belongs to
		unsigned int index;					//iterators can not be stored
									//however, most of the code is not duplicated
		virtual void set() = 0;
	
	public:
		
		ArrAttrItem(const vector<CLASS>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m):  array(array_ref), index (a_i), meta(m) 
		{
			object = oi;
			if (!object)
				throw udm_exception("Object pointer is NULL");
		};
		operator CLASS() const 
		{
			if (array.size() > index) return array.at(index); 
			else return CLASS();
		};

		CLASS operator*(){return (CLASS)*this;};
		

		virtual operator =(const CLASS &a)
		{
			//pad with CLASS() if array is smaller than index
			while (array.size() <=index)
				array.push_back(CLASS());

			CLASS & i = (CLASS&)array.at(index);
			i = a;
			set();
		};


		virtual bool operator==(const ArrAttrItem<CLASS>& a)
		{
			if (array != a.array)
				throw udm_exception("Comparision of two unrelated iterators!");
			
			return (index == a.index);
		};

		virtual bool operator!=(const ArrAttrItem<CLASS>& a)
		{
			
			return !(*this == a);
		};




		//iterator-like mechanism, inline postfix routine
		virtual operator++(int){index++;};

		
		void CheckOrdering()
		{
			if (CheckAttributeOrdering(meta))
					throw udm_exception("Items at a specified position in ordered attributes can not be set directly!");
		}
			
		

	};

	/*
		Array element template class which also implements the
		+= operator for long, string, real types
	*/
	template <class CLASS>
	class AdditiveArrAttrItem : public ArrAttrItem<CLASS>
	{
	public:
		AdditiveArrAttrItem(const vector<CLASS>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:ArrAttrItem<CLASS>(array_ref, a_i, oi, m){};
		
		virtual operator +=(const CLASS &a)
		{
			//pad with CLASS() if array is smaller than index
			while (array.size() <=index)
				array.push_back(CLASS());

			CLASS & i = array.at(index);
			i += a;
			set();
		};

		virtual operator = (const CLASS &a)
		{
			ArrAttrItem<CLASS>::operator =(a);
		}
	};

	//persistent array attribute element(iterator) classes
	// string array element providing operations with const char *
	class StringArrAttrItem : public AdditiveArrAttrItem<string>
	{
	public:	
		StringArrAttrItem(const vector<string>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:AdditiveArrAttrItem<string>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setStringAttrArr(meta, array);
		};
		
		//provide these operators for const char * as well
		virtual operator =(const char * a)
		{
			AdditiveArrAttrItem<string>::operator=(string(a));
		};

		virtual operator += (const char * a)
		{
			AdditiveArrAttrItem<string>::operator +=(string(a));
		};
		
	};

	class RealArrAttrItem : public AdditiveArrAttrItem<double>
	{
	public:	
		RealArrAttrItem(const vector<double>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:AdditiveArrAttrItem<double>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setRealAttrArr(meta, array);
		};
		
		//provide these operators for long, int, float, char as well
		operator =(const int a)		{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const int  a)	{AdditiveArrAttrItem<double>::operator +=((double)a);};
		operator = (const long  a)	{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const long  a)	{AdditiveArrAttrItem<double>::operator +=((double)a);};
		operator = (const char  a)	{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const char  a)	{AdditiveArrAttrItem<double>::operator +=((double)a);};
		operator = (const float  a)	{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const float  a){AdditiveArrAttrItem<double>::operator +=((double)a);};
		
	};

	class IntegerArrAttrItem : public AdditiveArrAttrItem<__int64>
	{
	public:
		IntegerArrAttrItem(const vector<__int64>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:AdditiveArrAttrItem<__int64>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setIntegerAttrArr(meta, array);
		};
		
		//provide these operators for long, int, float, char as well
		virtual operator =(const double a)		{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const double  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		virtual operator = (const int  a)	{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const int  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		virtual operator = (const char  a)	{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const char  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		virtual operator = (const float  a)	{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const float  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		
	};

	class BooleanArrAttrItem : public ArrAttrItem<bool>
	{
	public:
		BooleanArrAttrItem(const vector<bool>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:ArrAttrItem<bool>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setBooleanAttrArr(meta, array);
		};
		
		//provide these operators for long, int, float, char as well
		virtual operator =(const double a)	{ArrAttrItem<bool>::operator =(a != 0.0);};
		virtual operator = (const long  a)	{ArrAttrItem<bool>::operator =(a != 0);};
		virtual operator = (const char  a)	{ArrAttrItem<bool>::operator =(a != 0);};
		virtual operator = (const float  a)	{ArrAttrItem<bool>::operator =(a != (float)0.0);};
		
	};

	


	//non-persistent, array attribute iterators

		//persistent array attribute element(iterator) classes
	// string array element providing operations with const char *
	class TempStringArrAttrItem : public AdditiveArrAttrItem<string>
	{
	public:	
		TempStringArrAttrItem(const vector<string>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:AdditiveArrAttrItem<string>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setTempStringAttrArr(meta, array);
		};
		
		//provide these operators for const char * as well
		virtual operator =(const char * a)
		{
			AdditiveArrAttrItem<string>::operator=(string(a));
		};

		virtual operator += (const char * a)
		{
			AdditiveArrAttrItem<string>::operator +=(string(a));
		};
		
	};

	class TempRealArrAttrItem : public AdditiveArrAttrItem<double>
	{
	public:	
		TempRealArrAttrItem(const vector<double>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:AdditiveArrAttrItem<double>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setTempRealAttrArr(meta, array);
		};
		
		//provide these operators for long, int, float, char as well
		operator =(const int a)		{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const int  a)	{AdditiveArrAttrItem<double>::operator +=((double)a);};
		operator = (const long  a)	{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const long  a)	{AdditiveArrAttrItem<double>::operator +=((double)a);};
		operator = (const char  a)	{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const char  a)	{AdditiveArrAttrItem<double>::operator +=((double)a);};
		operator = (const float  a)	{AdditiveArrAttrItem<double>::operator =((double)a);};
		operator += (const float  a){AdditiveArrAttrItem<double>::operator +=((double)a);};
		
	};

	class TempIntegerArrAttrItem : public AdditiveArrAttrItem<__int64>
	{
	public:
		TempIntegerArrAttrItem(const vector<__int64>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:AdditiveArrAttrItem<__int64>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setTempIntegerAttrArr(meta, array);
		};
		
		//provide these operators for long, int, float, char as well
		virtual operator =(const double a)		{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const double  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		virtual operator = (const int  a)	{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const int  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		virtual operator = (const char  a)	{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const char  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		virtual operator = (const float  a)	{AdditiveArrAttrItem<__int64>::operator =((__int64)a);};
		virtual operator += (const float  a)	{AdditiveArrAttrItem<__int64>::operator +=((__int64)a);};
		
	};

	class TempBooleanArrAttrItem : public ArrAttrItem<bool>
	{
	public:
		TempBooleanArrAttrItem(const vector<bool>& array_ref, const int  a_i,  ObjectImpl * oi, const Uml::Attribute & m)
			:ArrAttrItem<bool>(array_ref, a_i, oi, m){};
		virtual void set()
		{
			CheckOrdering();		//this will crash if array is ordered
			//ordered attributes are not supposed to be changed in a by-item fashion

			object->setTempBooleanAttrArr(meta, array);
		};
		
		//provide these operators for long, int, float, char as well
		virtual operator =(const double a)	{ArrAttrItem<bool>::operator =(a != 0.0);};
		virtual operator = (const long  a)	{ArrAttrItem<bool>::operator =(a != 0);};
		virtual operator = (const char  a)	{ArrAttrItem<bool>::operator =(a != 0);};
		virtual operator = (const float  a)	{ArrAttrItem<bool>::operator =(a != (float)0.0);};
		
	};


	//non-persistent, array attribute objects

	

	template <class CLASS>
	vector<CLASS> SortArrAttr(const vector<CLASS> &a)
	{
		multiset<CLASS> b;
		vector<CLASS> c;
		b.insert(a.begin(), a.end());
		set<CLASS>::iterator i = b.begin();
		while (i != b.end()) c.push_back(*i++);
		return c;
	};

	vector<string> SortStrArrAttr(const vector<string> &a);

	/*
		These are needed because defvalue on Attribute is 
		always declared as a string array, but it's real type
		can be just about anything.
	*/

	vector<__int64> StrToLongArrAttr(const vector<string> &a);
	vector<double> StrToRealArrAttr(const vector<string> &a);
	vector<bool> StrToBoolArrAttr(const vector<string> &a);
	


	class TempStringAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempStringAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<string>() const { return impl->getTempStringAttrArr(meta); }
		const vector<string> &operator =(const vector<string> &a) 
		{ 
			if (CheckAttributeOrdering(meta))
				impl->setTempStringAttrArr(meta, SortStrArrAttr(a)); 
			else
				impl->setTempStringAttrArr(meta, a); 
			return a; 
		}

		//on write-operations the array is ordered if requested
		const vector<string> operator +=(const vector<string> &a) 
		{
			vector<string> b = impl->getTempStringAttrArr(meta);
			
			vector<string>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);

			//impl->setTempStringAttrArr(meta,b);	
			*this = b;
			return b;
		}
		
		const vector<string> operator +=(const string &a) 
		{
			vector<string> b = impl->getTempStringAttrArr(meta);
			b.push_back(a);
			//impl->setTempStringAttrArr(meta,b);	
			*this = b;
			return b;
		}

		const vector<string> operator +=(const char * a) 
		{
			vector<string> b = impl->getTempStringAttrArr(meta);
			b.push_back(string(a));
			//impl->setTempStringAttrArr(meta,b);	
			*this = b;
			return b;
		}

		bool operator !() const 
		{ 
			vector<string> b = impl->getTempStringAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<string> b = impl->getTempStringAttrArr(meta);
			return  (b.size() != 0); 
		};


		TempStringArrAttrItem  operator[](const int index)
		{
			vector<string> a = impl->getTempStringAttrArr(meta);
			TempStringArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

		TempStringArrAttrItem  begin()
		{
			vector<string> a = impl->getTempStringAttrArr(meta);
			TempStringArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}

		TempStringArrAttrItem  end()
		{
			vector<string> a = impl->getTempStringAttrArr(meta);
			TempStringArrAttrItem array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}


	};

	class TempIntegerAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempIntegerAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<__int64>() const { return impl->getTempIntegerAttrArr(meta); }
		const vector<__int64> &operator =(const vector<__int64> &a) 
		{ 
			
			if (CheckAttributeOrdering(meta))
				impl->setTempIntegerAttrArr(meta, SortArrAttr<__int64>(a)); 
			else
				impl->setTempIntegerAttrArr(meta, a); 
			return a; 
		}
		const vector<__int64> operator +=(const vector<__int64> &a) 
		{
			vector<__int64> b = impl->getTempIntegerAttrArr(meta);
			
			vector<__int64>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);
			//impl->setTempIntegerAttrArr(meta,b);	
			*this = b;
			return b;
		}


		const vector<__int64> operator +=(const long a) 
		{
			vector<__int64> b = impl->getTempIntegerAttrArr(meta);
			b.push_back(a);
			//impl->setTempIntegerAttrArr(meta,b);	
			*this = b;
			return b;
		}


		bool operator !() const 
		{ 
			vector<__int64> b = impl->getTempIntegerAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<__int64> b = impl->getTempIntegerAttrArr(meta);
			return  (b.size() != 0); 
		};

		TempIntegerArrAttrItem  operator[](const int index)
		{
			vector<__int64> a = impl->getTempIntegerAttrArr(meta);
			TempIntegerArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

		TempIntegerArrAttrItem  begin()
		{
			vector<__int64> a = impl->getTempIntegerAttrArr(meta);
			TempIntegerArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}
	
		TempIntegerArrAttrItem  end()
		{
			vector<__int64> a = impl->getTempIntegerAttrArr(meta);
			TempIntegerArrAttrItem array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}



	};


	class TempRealAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempRealAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<double>() const { return impl->getTempRealAttrArr(meta); }
		const vector<double> &operator =(const vector<double> &a) 
		{ 
			
			if (CheckAttributeOrdering(meta))
				impl->setTempRealAttrArr(meta, SortArrAttr<double>(a)); 
			else
				impl->setTempRealAttrArr(meta, a); 

			return a; 
		}
		const vector<double> operator +=(const vector<double> &a) 
		{
			vector<double> b = impl->getTempRealAttrArr(meta);
			
			vector<double>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);

			*this = b;    //call the above operator
			return b;
		}


		const vector<double> operator +=(const double a) 
		{
			vector<double> b = impl->getTempRealAttrArr(meta);
			b.push_back(a);
			*this = b;    //call the above operator
			return b;
		}


		bool operator !() const 
		{ 
			vector<double> b = impl->getTempRealAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<double> b = impl->getTempRealAttrArr(meta);
			return  (b.size() != 0); 
		};


		TempRealArrAttrItem  operator[](const int index)
		{
			vector<double> a = impl->getTempRealAttrArr(meta);
			TempRealArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

		TempRealArrAttrItem  begin()
		{
			vector<double> a = impl->getTempRealAttrArr(meta);
			TempRealArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}

		TempRealArrAttrItem  end()
		{
			vector<double> a = impl->getTempRealAttrArr(meta);
			TempRealArrAttrItem array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}

	};


	class TempBooleanAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		TempBooleanAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<bool>() const { return impl->getTempBooleanAttrArr(meta); }
		const vector<bool> &operator =(const vector<bool> &a) 
		{ 
			if (CheckAttributeOrdering(meta))
				impl->setTempBooleanAttrArr(meta, SortArrAttr(a));
			else
				impl->setTempBooleanAttrArr(meta, a); 
			return a; 
		}

		const vector<bool> operator +=(const vector<bool> &a) 
		{
			vector<bool> b = impl->getTempBooleanAttrArr(meta);
			
			vector<bool>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);
			//impl->setTempBooleanAttrArr(meta,b);	
			*this = b;
			return b;
		}


		const vector<bool> operator +=(const bool a) 
		{
			vector<bool> b = impl->getTempBooleanAttrArr(meta);
			b.push_back(a);
			//impl->setTempBooleanAttrArr(meta,b);	
			*this = b;
			return b;
		}


		bool operator !() const 
		{ 
			vector<bool> b = impl->getTempBooleanAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<bool> b = impl->getTempBooleanAttrArr(meta);
			return  (b.size() != 0); 
		};


		TempBooleanArrAttrItem  operator[](const int index)
		{
			vector<bool> a = impl->getTempBooleanAttrArr(meta);
			TempBooleanArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

		TempBooleanArrAttrItem  begin()
		{
			vector<bool> a = impl->getTempBooleanAttrArr(meta);
			TempBooleanArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}

		TempBooleanArrAttrItem  end()
		{
			vector<bool> a = impl->getTempBooleanAttrArr(meta);
			TempBooleanArrAttrItem array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}

	};



// --------------------------- StringAttrArr

	class StringAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		StringAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<string>() const { return impl->getStringAttrArr(meta); }
		const vector<string> &operator =(const vector<string> &a) 
		{ 
			if (CheckAttributeOrdering(meta))
				impl->setStringAttrArr(meta, SortStrArrAttr(a));
			else
				impl->setStringAttrArr(meta, a); 

			return a; 

		}
		const vector<string> operator +=(const vector<string> &a) 
		{
			vector<string> b = impl->getStringAttrArr(meta);
			
			vector<string>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);
			//impl->setStringAttrArr(meta,b);	
			*this = b;
			return b;
		}
		
		const vector<string> operator +=(const string &a) 
		{
			vector<string> b = impl->getStringAttrArr(meta);
			b.push_back(a);
			//impl->setStringAttrArr(meta,b);	
			*this = b;
			return b;
		}

		const vector<string> operator +=(const char * a) 
		{
			vector<string> b = impl->getStringAttrArr(meta);
			b.push_back(string(a));
			//impl->setStringAttrArr(meta,b);	
			*this = b;
			return b;

		}

		bool operator !() const 
		{ 
			vector<string> b = impl->getStringAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<string> b = impl->getStringAttrArr(meta);
			return  (b.size() != 0); 
		};


		StringArrAttrItem  operator[](const int index)
		{
			vector<string> a = impl->getStringAttrArr(meta);
			StringArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}


		StringArrAttrItem  begin()
		{
			vector<string> a = impl->getStringAttrArr(meta);
			StringArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}

		StringArrAttrItem  end()
		{
			vector<string> a = impl->getStringAttrArr(meta);
			StringArrAttrItem array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}
	
		
		
	};

// --------------------------- BooleanAttrArr

	class BooleanAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		BooleanAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<bool>() const { return impl->getBooleanAttrArr(meta); }
		const vector<bool>& operator =(const vector<bool>& a) 
		{ 
			if (CheckAttributeOrdering(meta))
				impl->setBooleanAttrArr(meta, SortArrAttr<bool>(a));
			else
				impl->setBooleanAttrArr(meta, a); 
			return a; 
		}

		const vector<bool> operator +=(const vector<bool> &a) 
		{
			vector<bool> b = impl->getBooleanAttrArr(meta);
			
			vector<bool>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);
			//impl->setBooleanAttrArr(meta,b);	
			*this = b;
			return b;
		}


		const vector<bool> operator +=(const bool a) 
		{
			vector<bool> b = impl->getBooleanAttrArr(meta);
			b.push_back(a);
			//impl->setBooleanAttrArr(meta,b);	
			*this = b;
			return b;
		}


		bool operator !() const 
		{ 
			vector<bool> b = impl->getBooleanAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<bool> b = impl->getBooleanAttrArr(meta);
			return  (b.size() != 0); 
		};


		BooleanArrAttrItem  operator[](const int index)
		{
			vector<bool> a = impl->getBooleanAttrArr(meta);
			BooleanArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

	};

// --------------------------- IntegerAttrArr

	class IntegerAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		IntegerAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<__int64>() const { return impl->getIntegerAttrArr(meta); }
		const vector<__int64>& operator =(const vector<__int64>& a) 
		{
			if (CheckAttributeOrdering(meta))
				impl->setIntegerAttrArr(meta, SortArrAttr<__int64>(a)); 
			else
				impl->setIntegerAttrArr(meta, a); 

			return a; 
		}
		const vector<__int64> operator +=(const vector<__int64> &a) 
		{
			vector<__int64> b = impl->getIntegerAttrArr(meta);
			
			vector<__int64>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);
			//impl->setIntegerAttrArr(meta,b);	
			*this = b;
			return b;
		}


		const vector<__int64> operator +=(const __int64 a) 
		{
			vector<__int64> b = impl->getIntegerAttrArr(meta);
			b.push_back(a);
			//impl->setIntegerAttrArr(meta,b);	
			*this = b;
			return b;
		}


		bool operator !() const 
		{ 
			vector<__int64> b = impl->getIntegerAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<__int64> b = impl->getIntegerAttrArr(meta);
			return  (b.size() != 0); 
		};

		IntegerArrAttrItem  operator[](const int index)
		{
			vector<__int64> a = impl->getIntegerAttrArr(meta);
			IntegerArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

		IntegerArrAttrItem begin()
		{
			vector<__int64> a = impl->getIntegerAttrArr(meta);
			IntegerArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}

		IntegerArrAttrItem end()
		{
			vector<__int64> a = impl->getIntegerAttrArr(meta);
			IntegerArrAttrItem array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}

	};

// --------------------------- RealAttrArr

	class RealAttrArr
	{
	private:
		ObjectImpl *impl;
		const Uml::Attribute &meta;

	public:
		RealAttrArr(ObjectImpl *i, const Uml::Attribute &m) : impl(i), meta(m) { }

		operator vector<double>() const { return impl->getRealAttrArr(meta); }
		const vector<double>& operator =(const vector<double> &a) 
		{
			if (CheckAttributeOrdering(meta))
				impl->setRealAttrArr(meta, SortArrAttr<double>(a)); 
			else
				impl->setRealAttrArr(meta, a); 
			return a; 
		}

		const vector<double> operator +=(const vector<double> &a) 
		{
			vector<double> b = impl->getRealAttrArr(meta);
			
			vector<double>::const_iterator ci_a = a.begin();
			while (ci_a != a.end())
				b.push_back(*ci_a++);
			//impl->setRealAttrArr(meta,b);	
			*this = b;
			return b;
		}


		const vector<double> operator +=(const double a) 
		{
			vector<double> b = impl->getRealAttrArr(meta);
			b.push_back(a);
			//	impl->setRealAttrArr(meta,b);	
			*this = b;
			return b;
		}


		bool operator !() const 
		{ 
			vector<double> b = impl->getRealAttrArr(meta);
			return  (b.size() == 0);
		};
		operator bool()  const 
		{ 
			vector<double> b = impl->getRealAttrArr(meta);
			return  (b.size() != 0); 
		};


		RealArrAttrItem  operator[](const int index)
		{
			vector<double> a = impl->getRealAttrArr(meta);
			RealArrAttrItem array_indexer(a, index, impl, meta);
			return array_indexer;
		}

		RealArrAttrItem begin()
		{
			vector<double> a = impl->getRealAttrArr(meta);
			RealArrAttrItem array_indexer(a, 0, impl, meta);
			return array_indexer;
		}

		RealArrAttrItem end()
		{
			vector<double> a = impl->getRealAttrArr(meta);
			RealArrAttrItem  array_indexer(a, a.size(), impl, meta);
			return array_indexer;
		}


	};

// --------------------------- AssocAttr

	template<class CLASS>
	class AssocAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::AssociationRole &meta;

	public:
		AssocAttr(ObjectImpl *i, const Uml::AssociationRole &m) : impl(i), meta(m) { }

		operator set<CLASS>() const 
		{
			set<CLASS> ret;

			vector<ObjectImpl*> a = impl->getAssociation(meta);
			vector<ObjectImpl*>::iterator i = a.begin();
			while( i != a.end() )
			{
				// TODO: assert the type of the object
				ret.insert(*i);
				++i;
			}

			return ret;
		}

		const set<CLASS> &operator =(const set<CLASS> &a)
		{
			vector<ObjectImpl*> b;

			set<CLASS>::const_iterator i = a.begin();
			while( i != a.end() )
			{
				b.push_back((*i).__impl());
				++i;
			}

			impl->setAssociation(meta, b);

			return a;
		}

		const set<CLASS> operator +=(const CLASS &a) {
			set<CLASS> xx(*this);
			xx.insert(a);
			return *this = xx;
		}
		const set<CLASS> operator -=(const CLASS &a) {
			set<CLASS> xx(*this);
			xx.erase(a);
			return *this = xx;
		}
	};

// --------------------------- PointerAttr

	template<class CLASS>
	class PointerAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::AssociationRole &meta;

	public:
		PointerAttr(ObjectImpl *i, const Uml::AssociationRole &m) : impl(i), meta(m) { }

		operator CLASS() const 
		{
			vector<ObjectImpl*> a = impl->getAssociation(meta);
			ASSERT( a.size() <= 1 );

			// TODO: assert the type of the object
			return a.empty() ? &_null : a.front();
		}

		const CLASS &operator =(const CLASS &a)
		{
			vector<ObjectImpl*> b;

			if( a != null )
				b.push_back(a.__impl());

			impl->setAssociation(meta, b);

			return a;
		}
	};


// --------------------------- AssocEndAttr

	template<class CLASS>
	class AssocEndAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::AssociationRole &meta;

	public:
		AssocEndAttr(ObjectImpl *i, const Uml::AssociationRole &m) : impl(i), meta(m) { }

		operator CLASS() const 
		{
			vector<ObjectImpl*> a = impl->getAssociation(meta, TARGETFROMCLASS);
			ASSERT( a.size() <= 1 );

			// TODO: assert the type of the object
			return a.empty() ? &_null : a.front();
		}

		const CLASS &operator =(const CLASS &a)
		{
			vector<ObjectImpl*> b;

			if( a != null )
				b.push_back(a.__impl());

			impl->setAssociation(meta, b, TARGETFROMCLASS);

			return a;
		}
	};

// --------------------------- AClassPointerAttr

	template<class CLASS, class TARGETCLASS>
	class AClassPointerAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::AssociationRole &peermeta, &selfmeta;

	public:
		AClassPointerAttr(ObjectImpl *i, const Uml::AssociationRole &m, const Uml::AssociationRole &m2) : impl(i), peermeta(m), selfmeta(m2) { }

		operator CLASS() const 
		{
			vector<ObjectImpl*> a = impl->getAssociation(peermeta, CLASSFROMTARGET);
			ASSERT( a.size() <= 1 );

			// TODO: assert the type of the object
			return a.empty() ? &_null : a.front();
		}

		const CLASS &operator =(const CLASS &a)
		{
			vector<ObjectImpl*> b;

			if( a != null )
				b.push_back(a.__impl());

			impl->setAssociation(peermeta, b, CLASSFROMTARGET);

			return a;
		}

		CLASS SetLink(TARGETCLASS peer, Object parent, 
			const Uml::Class &meta = CLASS::meta,
			const Uml::CompositionChildRole &role = NULLCHILDROLE) {
				
			if(!IsDerivedFrom(meta, CLASS::meta)) {
						throw udm_exception("Invalid type for assoc class");
			}

			CLASS nn = CLASS::Cast(Object::Create(meta, parent, role));
			AssocEndAttr<TARGETCLASS> zz = AssocEndAttr<TARGETCLASS>(nn.__impl(), peermeta);
			zz = peer;
			*this = nn;
			return nn;
		}	
	};

// --------------------------- AClassAssocAttr

	template<class CLASS, class TARGETCLASS>
	class AClassAssocAttr
	{
	private:
		ObjectImpl *impl;
// selfmeta is the role pointing towards the owner
// the peer points to the other
		const Uml::AssociationRole &peermeta, &selfmeta;

	public:
		AClassAssocAttr(ObjectImpl *i, const Uml::AssociationRole &m, const Uml::AssociationRole &m2) : impl(i), peermeta(m), selfmeta(m2) { }

		operator set<CLASS>() const 
		{
			set<CLASS> ret;

			vector<ObjectImpl*> a = impl->getAssociation(peermeta, CLASSFROMTARGET);
			vector<ObjectImpl*>::iterator i = a.begin();
			while( i != a.end() )
			{
				ret.insert(*i);
				++i;
			}

			return ret;
		}

		const set<CLASS> &operator =(const set<CLASS> &a)
		{
			vector<ObjectImpl*> b;

			set<CLASS>::const_iterator i = a.begin();
			while( i != a.end() )
			{
				b.push_back((*i).__impl());
				++i;
			}

			impl->setAssociation(peermeta, b, CLASSFROMTARGET);

			return a;
		}

		CLASS AddLink(TARGETCLASS peer, Object parent, 
			const Uml::Class &meta = CLASS::meta,
			const Uml::CompositionChildRole &role = NULLCHILDROLE) {
				
			if(!IsDerivedFrom(meta, CLASS::meta)) {
						throw udm_exception("Invalid type for assoc class");
			}

			CLASS nn = CLASS::Cast(Object::Create(meta, parent, role));
			AssocEndAttr<TARGETCLASS> zz = AssocEndAttr<TARGETCLASS>(nn.__impl(), peermeta);
			zz = peer;
			set<CLASS> b = *this;
			b.insert(nn);
			*this = b;
			return nn;
		}	
	};

// --------------------------- ChildrenAttr

	template<class CLASS>
	class ChildrenAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::CompositionChildRole &meta;

	public:
		ChildrenAttr(ObjectImpl *i, const Uml::CompositionChildRole &m) : impl(i), meta(m) { }

		operator set<CLASS>() const 
		{
			set<CLASS> ret;

			vector<ObjectImpl*> a = impl->getChildren(meta, CLASS::meta);
			vector<ObjectImpl*>::iterator i = a.begin();
			while( i != a.end() )
			{
				// TODO: assert the type of the object
				ret.insert(*i);
				++i;
			}

			return ret;
		}

		operator vector<CLASS>() const 
		{
			vector<CLASS> ret;

			vector<ObjectImpl*> a = impl->getChildren(meta, CLASS::meta);
			vector<ObjectImpl*>::iterator i = a.begin();
			while( i != a.end() )
			{
				// TODO: assert the type of the object
				ret.push_back(*i);
				++i;
			}

			return ret;
		}



		const set<CLASS> &operator =(const set<CLASS> &a)
		{
			vector<ObjectImpl *> b;

			set<CLASS>::const_iterator i = a.begin();
			while( i != a.end() )
			{
				b.push_back((*i).__impl());
				++i;
			}

			impl->setChildren(meta, b);

			return a;
		}

		const vector<CLASS> &operator =(const vector<CLASS> &a)
		{
			vector<ObjectImpl *> b;
			set<CLASS> uniq;

			vector<CLASS>::const_iterator i = a.begin();
			while( i != a.end() )
			{
				pair<set<CLASS>::iterator, bool> ins_res = uniq.insert(*i);
				if (ins_res.second)
				{
					//object is unique
					b.push_back((*i).__impl());
					++i;
				}
			}

			impl->setChildren(meta, b);

			return a;
		}






		const vector<CLASS> operator +=(const CLASS &a) 
		{
			//set<CLASS> xx(*this);
			//xx.insert(a);
			vector<CLASS> xx(*this);
			xx.push_back(a);
			return *this = xx;
			
		}

		const vector<CLASS> operator -=(const CLASS &a) 
		{
			//set<CLASS> xx(*this);
			//xx.erase(a);
			vector<CLASS> xx(*this);
			vector<CLASS>::iterator i = xx.begin();
			while ( i != xx.end())
			{
				if (*i == a)
				{
					i = xx.erase(i);
				}
				else
					++i;
			}
			return *this = xx;
		}
	};

// --------------------------- ChildAttr

	template<class CLASS>
	class ChildAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::CompositionChildRole &meta;

	public:
		ChildAttr(ObjectImpl *i, const Uml::CompositionChildRole &m) : impl(i), meta(m) { }

		operator CLASS() const 
		{
			vector<ObjectImpl*> a = impl->getChildren(meta, CLASS::meta);
			ASSERT( a.size() <= 1 );

			// TODO: assert the type of the object
			return a.empty() ? &_null : a.front();
		}

		const CLASS &operator =(const CLASS &a)
		{
			vector<ObjectImpl*> b;

			if( a != null )
				b.push_back(a.__impl());

			impl->setChildren(meta, b);

			return a;
		}
	};

// --------------------------- ParentAttr

	template<class CLASS>
	class ParentAttr
	{
	private:
		ObjectImpl *impl;
		const Uml::CompositionParentRole &meta;

	public:
		ParentAttr(ObjectImpl *i, const Uml::CompositionParentRole &m) : impl(i), meta(m) { }

		operator CLASS() const {
			return CLASS::Cast(impl->getParent(meta));
		}

		// we should speed this up using some iterator class
		const CLASS &operator =(const CLASS &a)
		{
// CYT4 - to allow the effective removal of node by setting parent to null
//			ASSERT( a != null );

			impl->setParent(a.__impl(), meta);

			return a;
		}
	};
// -----------------------------Typed set<> container

	template<class CLASS>
	class DerivedAttr
	{
		private:
			ObjectImpl *impl;
		public:
			DerivedAttr(ObjectImpl *i) : impl(i) {}
			operator set<CLASS>() const 
			{
				set<CLASS> ret;
				vector<ObjectImpl*> a = impl->getDerived();
				vector<ObjectImpl*>::iterator i = a.begin();
				while( i != a.end() )
				{
					// TODO: assert the type of the object
					ret.insert(*i);
					++i;
				}

				return ret;
			}
	};


	template<class CLASS>
	class InstantiatedAttr
	{
		private:
			ObjectImpl *impl;
		public:
			InstantiatedAttr(ObjectImpl *i) : impl(i) {}
			operator set<CLASS>() const 
			{
				set<CLASS> ret;
				vector<ObjectImpl*> a = impl->getInstances();
				vector<ObjectImpl*>::iterator i = a.begin();
				while( i != a.end() )
				{
					// TODO: assert the type of the object
					ret.insert(*i);
					++i;
				}

				return ret;
			}
	};

	template<class CLASS>
	class ArchetypeAttr
	{
		private:
			ObjectImpl * impl;
		public:
			ArchetypeAttr(ObjectImpl *i) : impl(i) {}
			operator CLASS() const
			{
				return CLASS::Cast(impl->getArchetype());
			};
	};

// --------------------------- Object

	class Object
	{
	// --- friends

		friend void Initialize();
		friend void Terminate();
		
	protected:
		ObjectImpl *impl;	// this is never a NULL pointer

	public:
		ObjectImpl *__impl() const { return impl; }

	// --- construction

	public:
		Object() : impl(&_null) { ASSERT(impl != NULL); }
		Object(ObjectImpl *i) : impl(i != NULL ? i : &_null) { ASSERT(impl != NULL); }
		
		Object(const Object &a) : impl(a.impl->clone()) { ASSERT(impl != NULL); }
		~Object() 
		{
			ASSERT(impl != NULL);
			impl->release(); 
#ifdef _DEBUG
			impl = NULL;
#endif
		}

		const Object &operator =(const Object &a)
		{
			ASSERT(a.impl != NULL);
			ObjectImpl *t = a.impl->clone();

			ASSERT(impl != NULL);
			impl->release();

			impl = t;
			return a;
		}

		static Object Cast(const Object &a, const Uml::Class &meta) { return __Cast(a, meta); }
		static Object Create(const Uml::Class &meta, const Object &parent, 
			const Uml::CompositionChildRole &role, 
			const ObjectImpl * archetype = &_null,
			const bool subtype = false) 
		{ 
			return  __Create(meta, parent, role, archetype, subtype); 
		}

	protected:
		static ObjectImpl *__Cast(const Object &a, const Uml::Class &meta);
		static ObjectImpl *__Create(const Uml::Class &meta, const Object &parent,
			const Uml::CompositionChildRole &role,
			const ObjectImpl *  archetype = &_null,
			const bool subtype = false);

	
	public:
		// --- non-array/single value attributes

		string getStringAttr(const Uml::Attribute &meta) const { return impl->getStringAttr(meta); }
		void setStringAttr(const Uml::Attribute &meta, const string &a) { impl->setStringAttr(meta, a); }

		bool getBooleanAttr(const Uml::Attribute &meta) const { return impl->getBooleanAttr(meta); }
		void setBooleanAttr(const Uml::Attribute &meta, bool a) { impl->setBooleanAttr(meta, a); }

		__int64 getIntegerAttr(const Uml::Attribute &meta) const { return impl->getIntegerAttr(meta); }
		void setIntegerAttr(const Uml::Attribute &meta, __int64 a) { impl->setIntegerAttr(meta, a); }

		double getRealAttr(const Uml::Attribute &meta) const { return impl->getRealAttr(meta); }
		void setRealAttr(const Uml::Attribute &meta, double a) { impl->setRealAttr(meta, a); }

		// --- multiple value /array attributes
		vector<string> getStringAttrArr(const Uml::Attribute &meta) const { return impl->getStringAttrArr(meta); }
		void setStringAttrArr(const Uml::Attribute &meta, const vector<string> &a) { impl->setStringAttrArr(meta, a); }

		vector<bool> getBooleanAttrArr(const Uml::Attribute &meta) const { return impl->getBooleanAttrArr(meta); }
		void setBooleanAttrArr(const Uml::Attribute &meta, const vector<bool> &a) { impl->setBooleanAttrArr(meta, a); }

		vector<__int64> getIntegerAttrArr(const Uml::Attribute &meta) const { return impl->getIntegerAttrArr(meta); }
		void setIntegerAttrArr(const Uml::Attribute &meta, const vector<__int64> &a) { impl->setIntegerAttrArr(meta, a); }

		vector<double> getRealAttrArr(const Uml::Attribute &meta) const { return impl->getRealAttrArr(meta); }
		void setRealAttrArr(const Uml::Attribute &meta, const vector<double> &a) { impl->setRealAttrArr(meta, a); }



	// --- associations

	public:
/*
		template
		class AssocClass

  a.bs()
  a.bs_Classes().Class
  a.bs_Classes().bs_end
  a.bs_Classes().as_end
*/
		set<Object> getAssociation(const Uml::AssociationRole &meta) const
		{
			set<Object> ret;

			vector<ObjectImpl*> a = impl->getAssociation(meta);
			vector<ObjectImpl*>::const_iterator i = a.begin();
			while( i != a.end() )
			{
				ret.insert(*i);
				++i;
			}

			return ret;
		}

		

		void setAssociation(const Uml::AssociationRole &meta, const set<Object> &a)
		{
			vector<ObjectImpl*> b;

			set<Object>::const_iterator i = a.begin();
			while( i != a.end() )
			{
				b.push_back((*i).impl);
				++i;
			}

			impl->setAssociation(meta, b);
		}

	// --- type

	public:
		const Uml::Class &type() const { return impl->type(); }

	// --- order

	public:
		typedef ObjectImpl::uniqueId_type uniqueId_type;

		uniqueId_type uniqueId() const { return impl->uniqueId(); }

		bool operator ==(const Object &a) const { return uniqueId() == a.uniqueId(); }
		bool operator !=(const Object &a) const { return uniqueId() != a.uniqueId(); }
		bool operator <(const Object &a) const { return uniqueId() < a.uniqueId(); }
		bool operator >(const Object &a) const { return uniqueId() > a.uniqueId(); }
		bool operator <=(const Object &a) const { return uniqueId() <= a.uniqueId(); }
		bool operator >=(const Object &a) const { return uniqueId() >= a.uniqueId(); }
		bool operator !() const { return uniqueId() == 0; }
		operator bool()  const { return  uniqueId() != 0; }

		// UDM TOMI Paradigm Independent Interface
		struct AssociationInfo
		{
			// clsAssociation can be empty (ClassType(NULL)) - simple association.
			const Uml::Class & clsAssociation;
			string strSrcRoleName;
			string strDstRoleName;
			AssociationInfo(const Uml::Class & ref) : clsAssociation(ref){};
		};


		// UDM TOMI Paradigm Independent Interface
		struct CompositionInfo
		{	
			string strParentRoleName;
			string strChildRoleName;
		};
		// UDM TOMI Paradigm Independent Interface
		// Retrieves the adjacent objects of an object associated via simple association or association class. 
		// The returned set can be empty. Composition relationships are not considered here.
		set<Object> GetAdjacentObjects();
		// UDM TOMI Paradigm Independent Interface
		// Retrieves the adjacent objects of an object. The adjacent objects 
		// are of the type of clsType or derived from it. The returned set can 
		// be empty. Composition relationships are not considered here.
		set<Object> GetAdjacentObjects(const Uml::Class & clsDstType);
		// UDM TOMI Paradigm Independent Interface
		// Retrieves the adjacent objects of an object via link instance of ascType. 
		// The adjacent objects are of the type of clsType or derived from it. 
		// The returned set can be empty. Composition relationships are not considered here. 
		// Parameter clsType can be null.
		set<Object> GetAdjacentObjects(const Uml::Class & clsDstType, const AssociationInfo& ascType);
		// UDM TOMI Paradigm Independent Interface
		// Get attributes by name. These are INEFFICIENT functions using iteration.
		// These functions return false if the attribute with the specified type
		// and attribute name does not exist. Hence these parameters are specified
		// in the metamodel, it can be serious error. If no problem they retrieve true.
		bool GetIntValue(string strAttrName, __int64& value);
		// UDM TOMI Paradigm Independent Interface
		// get the value of Attribute named strAttrName of type String
		// return false if not found,true if ok
		bool GetStrValue( string strAttrName, string& value);
		// UDM TOMI Paradigm Independent Interface
		// get the value of Attribute named strAttrName of type Real
		// return false if not found,true if ok
		bool GetRealValue( string strAttrName, double& value);
		// UDM TOMI Paradigm Independent Interface
		// get the value of Attribute named strAttrName of type Bool
		// return false if not found,true if ok
		bool GetBoolValue( string strAttrName, bool& value);
		// UDM TOMI Paradigm Independent Interface
		// get the value of Attribute named strAttrName of type Int
		// return false if not found,true if ok
		bool SetIntValue( string strAttrName, long value);
		// UDM TOMI Paradigm Independent Interface
		// set the value of Attribute named strAttrName of type String
		// return false if not found,true if ok
		bool SetStrValue(string strAttrName, const string& value);
		// UDM TOMI Paradigm Independent Interface
		// set the value of Attribute named strAttrName of type Real
		// return false if not found,true if ok
		bool SetRealValue( string strAttrName, double value);
		// UDM TOMI Paradigm Independent Interface
		// set the value of Attribute named strAttrName of type Bool
		// return false if not found,true if ok
		bool SetBoolValue( string strAttrName, bool value);
		// UDM TOMI Paradigm Independent Interface
		// Functions for composition
		// Returns the parent object.
		Object GetParent() const;
		// UDM TOMI Paradigm Independent Interface
		// Retrieves all children not considering types and role names.
		set<Object> GetChildObjects() const;
		// UDM TOMI Paradigm Independent Interface
		// Retrieves all children considering child types but not role names.
		set<Object> GetChildObjects(const Uml::Class & clsType);
		// UDM TOMI Paradigm Independent Interface
		// Retrieves all children considering role names and child types.
		// To ignore child types set clsChildType to ClassType(NULL).
		set<Object> GetChildObjects(const CompositionInfo& cmpType, const Uml::Class & clsChildType);
		
		// Functions for associations and links
		// UDM TOMI Paradigm Independent Interface
		// Gets the objects of the association class from between two objects. 
		// ascType.clsAssociation  should be NULL to be ignored.
		set<Object> GetAssociationClassObjects(Object dstObject, const AssociationInfo& ascType);
		// UDM TOMI Paradigm Independent Interface
		// Gets the two peers from an object of association class type
		pair<Object,Object> GetPeersFromAssociationClassObject();
		// Functions for creating Udm entities
		// UDM TOMI Paradigm Independent Interface
		// Creates an object of clsType
		Object CreateObject(const Uml::Class & clsType);
		// UDM TOMI Paradigm Independent Interface
		// Creates a link of a simple association or an association with association class. 
		// If ascType.clsAssociation is not valid a simple association will be tried.
		// On error results in false, true otherwise.
		bool CreateLink(Object dstObject, const AssociationInfo& ascType);
		// Removes a link of a simple association or an association with association class
		// if AscType.clsAssociation is not valid a simple association will be attempted
		// On error or when there is no such link false, otherwise true is returned.	
		bool DeleteLink(Object dstObject, const AssociationInfo& ascType);
		// Removes an object from the persistent storage.
		void DeleteObject();
		//get one specific end of an association class object
		Object GetAssociationEnd(string roleName);




		//archetype/derived/instances relationship

		Object archetype() const;
		set<Object> derived() const;
		set<Object> instances() const;
		bool hasRealArchetype() const;


		//some tomi extension functions
		
		// find the (first, there should be only one) child (only first-level children)Object in 
		// where, which is an inherited subtype of this(it's there because the 
		// parent(where) is directly(Primary) derived)
		//throws exception if not found
		Object FindCorrespondingObjectInSubtypes(const Object & where, const bool prim_derived = false) const;


		// find the (first, there should be only one) child (only first-level children)Object in 
		// where, which is an inherited instance of this(it's there because the 
		// parent(where) is directly(Primary) instantiated)
		//throws exception if not found
		Object FindCorrespondingObjectInInstances(const Object & where, const bool prim_derived = false) const;


		// find the (first, there should be only one) child (recursive, whole three)Object in 
		// where, which is an inherited subtype of this(it's there because the 
		// parent(where) is directly(Primary) derived)
		//return Udm::_null, if none found
		Object reqFindCorrespondingObjectInSubtypes(const Object& where, const bool prim_derived = false) const;

		// find the (first, there should be only one) child (recursive, whole three)Object in 
		// where, which is an inherited instance of this(it's there because the 
		// parent(where) is directly(Primary) derived)
		//return Udm::_null, if none found
		Object reqFindCorrespondingObjectInInstances(const Object& where, const bool prim_derived = false) const;

		//functions which invoke the above 2 functions,
		//and throw exception if they return Udm::_null
		Object FindCorrespondingObjectInInstancesTree(const Object& where, const bool prim_derived = false) const;
		Object FindCorrespondingObjectInSubtypesTree(const Object& where, const bool prim_derived = false) const;

		//get the set of my parent's (primary derived)instances/subtypes
		set<Object> GetMyParentsSubtypes(const bool only_primary_derived = false) const;
		set<Object> GetMyParentsInstances(const bool only_primary_derived = false) const;	
		
		//check if an object has derived or instantiated objects
		bool HasObjectSubtypes(const bool only_primary_derived = false) const;
		bool HasObjectInstances(const bool only_primary_derived = false) const;
		
		//find the closest parent, which is primarily derived(has a real archetye);
		Object FindClosestPrimarilyDerivedParent() const;

		//finds a derived object in of this in where
		Object getDerivedObjectInTree(const Udm::Object &where) const;
		//check if which is within the same derived/instantiated block
		Object getDerivedObjectInPrimarilyDerivedBlock(const Udm::Object &where) const;


		//finds an instantied object in of this in where
		Object getInstantiatedObjectInTree(const Udm::Object &where) const;
		//check if which is within the same derived/instantiated block
		Object getInstantiatedObjectInPrimarilyDerivedBlock(const Udm::Object &where) const;


		Object getReferencedObject(const string refrole = "ref") const;

	};

	class udmvariant {
	public: 
		//simple attribute values
		udmvariant(const string &a) : strval(a) { ; }
		udmvariant(const __int64 &a) : longval(a) { ; }
		udmvariant(const double &a) : doubleval(a) { ; }
		udmvariant(const bool &a) : boolval(a) { ; }

		//array attribute values
		udmvariant(const vector<string> &a) : strarrval(a) { ; }
		udmvariant(const vector<__int64> &a) : longarrval(a) { ; }
		udmvariant(const vector<double> &a) : doublearrval(a) { ; }
		udmvariant(const vector<bool> &a) : boolarrval(a) { ; }

		udmvariant(const Object &a) : objval(a) { ; }
		udmvariant(const set<Object> &a) : setval(a) { ; }
		//enum udmt type;			//what's this? 
		
		set<Object> setval;
		Object  objval;
		
		//simple attributes
		string strval;
		__int64    longval;
		double  doubleval;
		bool   boolval;
		//array attributes
		vector<string>	strarrval;
		vector<__int64>    longarrval;
		vector<double>  doublearrval;
		vector<bool>	boolarrval;
		

	};



// --------------------------- DataNetwork

enum BackendSemantics { 
	CHANGES_PERSIST_ALWAYS,		// committed sequences are recorded in original backend
								// no CloseNoUpdate,  CloseAs, SaveAs will be used
	CHANGES_PERSIST_DEFAULT,	// committed sequences are recorded in current backed unless
								// CloseNoUpdate,  CloseAs, or SaveAs is used
	CHANGES_LOST_DEFAULT };		// committed sequences are lost until 
								// Close, CloseAs, or SaveAs are used

struct UdmDiagram { 
		Uml::Diagram *dgr; 
		void (*init)(void);
};

class DataNetwork {
	friend ObjectImpl;
public:
	DataNetwork & operator=(DataNetwork &a); // enable assignment (was disable assignment)

protected:
// Non-persistent attributes
	ttmap tempvarmap;
	
	Object rootobject;
	const Udm::UdmDiagram &metaroot;
	DataNetwork(const Udm::UdmDiagram &metainfo);

public:
	typedef struct bt {
		string sig, ext;
		DataNetwork *(*crea)(const UdmDiagram &);
		struct bt *next;
	} backendtabt;
private: 
	static backendtabt *backendtab;
	static map <const unsigned long, DataNetwork *> dntab;
	static unsigned long dn_id_gen;
	unsigned long dn_id;
public:
	virtual ~DataNetwork() { if(rootobject) throw udm_exception("Dirty abort!"); }
// Load/Persist
	virtual void CreateNew(const string &systemname, 
									const string &metalocator, const Uml::Class &rootclass, 
									enum BackendSemantics sem = CHANGES_PERSIST_ALWAYS) = 0;
	virtual void OpenExisting(const string &systemname, 
									const string &metalocator = "", 
									enum BackendSemantics sem = CHANGES_PERSIST_ALWAYS) = 0;
	virtual void CloseWithUpdate() = 0;
	virtual void CloseNoUpdate()			{  throw udm_exception("Unsupported method"); }
	virtual void SaveAs(string systemname) {  throw udm_exception("Unsupported method"); }
	virtual void CloseAs(string systemname) {  throw udm_exception("Unsupported method"); }


	bool isOpen() { return rootobject; }
	unsigned long uniqueId(){return dn_id;};
// transaction control
	virtual void CommitEditSequence() = 0;
	virtual void AbortEditSequence() = 0;

// basic access
	Object GetRootObject() const { 
//		if(!rootobject) throw udm_exception("Null rootobject!!"); 
		return rootobject; 
	}
	virtual Object ObjectById(Object::uniqueId_type) { 
		throw udm_exception("This backend does not support ObjectByID!!"); 
	}
	const Uml::Diagram &GetRootMeta() const { 
		return *metaroot.dgr;
	}

	static void RegisterBackend(const string &sig, const string &ext, DataNetwork *(*crea)(const UdmDiagram &));
	static void UnRegisterBackends();
	static string DumpBackendNames();
	static DataNetwork *CreateBackend(string backendID, const UdmDiagram &metainfo);
	static DataNetwork *GetDnById(const unsigned long id);

//type safe
// StaticObjects do not copy the types in constructors
// They instead just reference it
// so care needs to be taken when using them
// DOM&GME are TypeSafe Data Networks
// we override this for Static and Smart
	virtual bool IsTypeSafe() {return true;};

};



class SmartDataNetwork : public DataNetwork 
{
	DataNetwork *dn;

public:
	//made public for CORBA to reach the inside Static DN
	DataNetwork *testdn() const {
		if(dn) return dn;
		throw udm_exception("Missing data network");
	}

	SmartDataNetwork(const UdmDiagram &metainfo) :
	  	DataNetwork(metainfo) {	dn = NULL;	}
		~SmartDataNetwork() 
		{ 
			delete dn;
		}
	
	SmartDataNetwork & operator=(SmartDataNetwork &a)
	{
		if (dn && a.dn)
		{
			*dn = *(a.dn);
		}
		else 
		{
			throw udm_exception("Either the source or the destination SmartDataNetwork does not exist.");
		}
		return *this;
	};

	
	virtual void CreateNew(const string &systemname, 
									const string &metalocator, const Uml::Class &rootclass, 
									enum BackendSemantics sem = CHANGES_PERSIST_ALWAYS) {
		if(dn) throw udm_exception("DataNetwork is already open"); 
		DataNetwork *dn1 = CreateBackend(systemname, metaroot);
		if(!dn1) throw udm_exception("Cannot deduce Udm backend type from " + systemname + "\n"
									 "Available backends: " + DumpBackendNames().c_str());
		dn1->CreateNew(systemname, metalocator, rootclass, sem);
		dn = dn1;
	}

	virtual void OpenExisting(const string &systemname, 
									const string &metalocator = "", 
									enum BackendSemantics sem = CHANGES_PERSIST_ALWAYS) {
		if(dn) throw udm_exception("DataNetwork is already open"); 
		DataNetwork *dn1 = CreateBackend(systemname, metaroot);
		if(!dn1) throw udm_exception("Cannot deduce Udm backend type from " + systemname + "\n"
									 "Available backends: " + DumpBackendNames().c_str());
		dn1->OpenExisting(systemname, metalocator, sem);
		dn = dn1;
	}
	virtual void CloseWithUpdate() { testdn()->CloseWithUpdate(); delete dn; dn = NULL;}
	virtual void CloseNoUpdate()	{  testdn()->CloseNoUpdate(); delete dn; dn = NULL; }
	virtual void SaveAs(string systemname) {  testdn()->SaveAs(systemname); }
	virtual void CloseAs(string systemname) {  testdn()->CloseAs(systemname); delete dn; dn = NULL;}

	bool isOpen() { return dn ? dn->isOpen() : false; }
// transaction control
	virtual void CommitEditSequence() { testdn()->CommitEditSequence(); }
	virtual void AbortEditSequence() { testdn()->AbortEditSequence(); }

// basic access
	Object GetRootObject() { return testdn()->GetRootObject(); }
	virtual Object ObjectById(Object::uniqueId_type t) { 
		return testdn()->ObjectById(t); 
	}
	const Uml::Diagram &GetRootMeta() const { 
		return testdn()->GetRootMeta(); 
	}
	virtual bool IsTypeSafe() { return testdn()->IsTypeSafe();};
};





}

//the biggest i64 integer is 20 characters long in decimal format.

inline ostream & operator<< (ostream &o, __int64 i) { char k[30];sprintf(k,"%I64d", i);	o << k; return o;};
inline ostream & operator<< (ostream &o, Udm::StringAttr c) { o << (string)c; return o; };

#endif//MOBIES_UDMBASE_H
